==> quantlib_0.9.0.20071224.orig.tar.gz: QuantLib-0.9.0/ql/models/marketmodels/products/multistep/multistepratchet.hpp <==
/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2006 Giorgio Facchinetti

 This file is part of QuantLib, a free-software/open-source library
 for financial quantitative analysts and developers - http://quantlib.org/

 QuantLib is free software: you can redistribute it and/or modify it
 under the terms of the QuantLib license.  You should have received a
 copy of the license along with this program; if not, please email
 <quantlib-dev@lists.sf.net>. The license is also available online at
 <http://quantlib.org/license.shtml>.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the license for more details.
*/

#ifndef quantlib_multistep_ratchet_hpp
#define quantlib_multistep_ratchet_hpp

#include <ql/models/marketmodels/products/multiproductmultistep.hpp>

namespace QuantLib {

    class MultiStepRatchet : public MultiProductMultiStep {
      public:
        MultiStepRatchet(const std::vector<Time>& rateTimes,
                         const std::vector<Real>& accruals,
                         const std::vector<Time>& paymentTimes,
                         Real gearingOfFloor,
                         Real gearingOfFixing,
                         Rate spreadOfFloor,
                         Rate spreadOfFixing,
                         Real initialFloor,
                         bool payer = true);
        //! \name MarketModelMultiProduct interface
        //@{
        std::vector<Time> possibleCashFlowTimes() const;
        Size numberOfProducts() const;
        Size maxNumberOfCashFlowsPerProductPerStep() const;
        void reset();
        bool nextTimeStep(
                     const CurveState& currentState,
                     std::vector<Size>& numberCashFlowsThisStep,
                     std::vector<std::vector<CashFlow> >& cashFlowsGenerated);
        std::auto_ptr<MarketModelMultiProduct> clone() const;
        //@}
      private:
        std::vector<Real> accruals_;
        std::vector<Time> paymentTimes_;
        Real gearingOfFloor_, gearingOfFixing_;
        Rate spreadOfFloor_, spreadOfFixing_;
        bool payer_;
        Real multiplier_;
        Size lastIndex_;
        // things that vary in a path
        Real floor_;
        Size currentIndex_;
    };


    // inline definitions

    inline std::vector<Time>
    MultiStepRatchet::possibleCashFlowTimes() const {
        return paymentTimes_;
    }

    inline Size MultiStepRatchet::numberOfProducts() const {
        return 1;
    }

    inline Size
    MultiStepRatchet::maxNumberOfCashFlowsPerProductPerStep() const {
        return 1;
    }

    inline void MultiStepRatchet::reset() {
       currentIndex_=0;
    }

}

#endif

==> quantlib_1.4.orig.tar.gz: QuantLib-1.4/ql/models/marketmodels/products/multistep/multistepratchet.hpp <==
/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2006 Giorgio Facchinetti

 This file is part of QuantLib, a free-software/open-source library
 for financial quantitative analysts and developers - http://quantlib.org/

 QuantLib is free software: you can redistribute it and/or modify it
 under the terms of the QuantLib license.  You should have received a
 copy of the license along with this program; if not, please email
 <quantlib-dev@lists.sf.net>. The license is also available online at
 <http://quantlib.org/license.shtml>.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the license for more details.
*/

#ifndef quantlib_multistep_ratchet_hpp
#define quantlib_multistep_ratchet_hpp

#include <ql/models/marketmodels/products/multiproductmultistep.hpp>

namespace QuantLib {

    class MultiStepRatchet : public MultiProductMultiStep {
      public:
        MultiStepRatchet(const std::vector<Time>& rateTimes,
                         const std::vector<Real>& accruals,
                         const std::vector<Time>& paymentTimes,
                         Real gearingOfFloor,
                         Real gearingOfFixing,
                         Rate spreadOfFloor,
                         Rate spreadOfFixing,
                         Real initialFloor,
                         bool payer = true);
        //! \name MarketModelMultiProduct interface
        //@{
        std::vector<Time> possibleCashFlowTimes() const;
        Size numberOfProducts() const;
        Size maxNumberOfCashFlowsPerProductPerStep() const;
        void reset();
        bool nextTimeStep(
                     const CurveState& currentState,
                     std::vector<Size>& numberCashFlowsThisStep,
                     std::vector<std::vector<CashFlow> >& cashFlowsGenerated);
        std::auto_ptr<MarketModelMultiProduct> clone() const;
        //@}
      private:
        std::vector<Real> accruals_;
        std::vector<Time> paymentTimes_;
        Real gearingOfFloor_, gearingOfFixing_;
        Rate spreadOfFloor_, spreadOfFixing_;
        bool payer_;
        Real multiplier_;
        Size lastIndex_;
        Real initialFloor_;
        // things that vary in a path
        Real floor_;
        Size currentIndex_;
    };


    // inline definitions

    inline std::vector<Time>
    MultiStepRatchet::possibleCashFlowTimes() const {
        return paymentTimes_;
    }

    inline Size MultiStepRatchet::numberOfProducts() const {
        return 1;
    }

    inline Size
    MultiStepRatchet::maxNumberOfCashFlowsPerProductPerStep() const {
        return 1;
    }

    inline void MultiStepRatchet::reset() 
    {
       currentIndex_=0;
       floor_ = initialFloor_;
    }

}

#endif
